import {
	CanActivate,
	ExecutionContext,
	Injectable,
	SetMetadata
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { Reflector } from '@nestjs/core';
import { User, UserRole } from './entities';

const IS_PUBLIC_KEY = 'isPublic';
export const Public = () => SetMetadata(IS_PUBLIC_KEY, true);

@Injectable()
export class JWTAuthGuard extends AuthGuard('jwt') {
	constructor(private reflector: Reflector) {
		super();
	}

	canActivate(context: ExecutionContext) {
		const isPublic = this.reflector.getAllAndOverride<boolean>(
			IS_PUBLIC_KEY,
			[context.getHandler(), context.getClass()]
		);
		if (isPublic) return true;
		return super.canActivate(context);
	}
}

const ROLE_KEY = 'role';
export const Role = (role: UserRole) => SetMetadata(ROLE_KEY, role);

@Injectable()
export class RoleGuard implements CanActivate {
	constructor(private reflector: Reflector) {}

	canActivate(context: ExecutionContext) {
		const requiredRole = this.reflector.getAllAndOverride<UserRole>(
			ROLE_KEY,
			[context.getHandler(), context.getClass()]
		);
		if (!requiredRole) return true;
		const user: User = context.switchToHttp().getRequest().user;
		return user.role >= requiredRole;
	}
}
